"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const NoRootTypeError_1 = require("./Error/NoRootTypeError");
const NodeParser_1 = require("./NodeParser");
const DefinitionType_1 = require("./Type/DefinitionType");
const symbolAtNode_1 = require("./Utils/symbolAtNode");
class SchemaGenerator {
    constructor(program, nodeParser, typeFormatter) {
        this.program = program;
        this.nodeParser = nodeParser;
        this.typeFormatter = typeFormatter;
        this.allTypes = new Map();
        const sourceFiles = this.program.getSourceFiles();
        this.prioritizedFiles = [];
        this.unprioritizedFiles = [];
        for (const f of sourceFiles) {
            if (!f.fileName.includes("/node_modules/")) {
                this.prioritizedFiles.push(f);
            }
            else {
                this.unprioritizedFiles.push(f);
            }
        }
    }
    createSchema(fullName) {
        const rootNode = this.findRootNode(fullName);
        const rootType = this.nodeParser.createType(rootNode, new NodeParser_1.Context());
        return Object.assign({ $schema: "http://json-schema.org/draft-06/schema#", definitions: this.getRootChildDefinitions(rootType) }, this.getRootTypeDefinition(rootType));
    }
    findRootNode(fullName) {
        const typeChecker = this.program.getTypeChecker();
        if (this.prioritizedFiles.length) {
            for (const sourceFile of this.prioritizedFiles) {
                this.inspectNode(sourceFile, typeChecker, this.allTypes);
            }
            this.prioritizedFiles = [];
        }
        if (this.allTypes.has(fullName)) {
            return this.allTypes.get(fullName);
        }
        if (this.unprioritizedFiles.length) {
            for (const sourceFile of this.unprioritizedFiles) {
                this.inspectNode(sourceFile, typeChecker, this.allTypes);
            }
            this.unprioritizedFiles = [];
        }
        if (this.allTypes.has(fullName)) {
            return this.allTypes.get(fullName);
        }
        throw new NoRootTypeError_1.NoRootTypeError(fullName);
    }
    inspectNode(node, typeChecker, allTypes) {
        if (node.kind === ts.SyntaxKind.InterfaceDeclaration ||
            node.kind === ts.SyntaxKind.EnumDeclaration ||
            node.kind === ts.SyntaxKind.TypeAliasDeclaration) {
            if (!this.isExportType(node)) {
                return;
            }
            else if (this.isGenericType(node)) {
                return;
            }
            allTypes.set(this.getFullName(node, typeChecker), node);
        }
        else {
            ts.forEachChild(node, (subnode) => this.inspectNode(subnode, typeChecker, allTypes));
        }
    }
    isExportType(node) {
        const localSymbol = symbolAtNode_1.localSymbolAtNode(node);
        return localSymbol ? "exportSymbol" in localSymbol : false;
    }
    isGenericType(node) {
        return !!(node.typeParameters &&
            node.typeParameters.length > 0);
    }
    getFullName(node, typeChecker) {
        const symbol = symbolAtNode_1.symbolAtNode(node);
        return typeChecker.getFullyQualifiedName(symbol).replace(/".*"\./, "");
    }
    getRootTypeDefinition(rootType) {
        return this.typeFormatter.getDefinition(rootType);
    }
    getRootChildDefinitions(rootType) {
        const seen = new Set();
        const children = this
            .typeFormatter.getChildren(rootType)
            .filter((child) => child instanceof DefinitionType_1.DefinitionType)
            .filter((child) => {
            if (!seen.has(child.getId())) {
                seen.add(child.getId());
                return true;
            }
            return false;
        });
        return children
            .reduce((result, child) => {
            const name = child.getName();
            if (name in result) {
                throw new Error(`Type "${name}" has multiple definitions.`);
            }
            return Object.assign({}, result, { [name]: this.typeFormatter.getDefinition(child.getType()) });
        }, {});
    }
}
exports.SchemaGenerator = SchemaGenerator;
//# sourceMappingURL=SchemaGenerator.js.map